import logging
from pytryfi.fiDevice import FiDevice
from pytryfi.common import query
from pytryfi.exceptions import *
from pytryfi.const import PET_ACTIVITY_ONGOINGWALK
import datetime

LOGGER = logging.getLogger(__name__)

class FiPet(object):
    def __init__(self, petId):
        self._petId = petId

    def setPetDetailsJSON(self, petJSON):
        try:
            self._name = petJSON['name']
            self._homeCityState = petJSON['homeCityState']
            self._yearOfBirth = int(petJSON['yearOfBirth'])
            self._monthOfBirth = int(petJSON['monthOfBirth'])
            self._dayOfBirth = int(petJSON['dayOfBirth'])
            self._gender = petJSON['gender']
            #weight is in kg
            self._weight = float(petJSON['weight'])
            self._breed = petJSON['breed']['name']
            #track last updated
            self._lastUpdated = datetime.datetime.now()
        except TryFiError as e:
            LOGGER.error(f"Unable to set values for Pet.\nException: {e}\nwhile parsing {petJSON}")
            raise TryFiError("Unable to set Pet Details")
        try:
            self._photoLink = petJSON['photos']['first']['image']['fullSize']
        except:
            LOGGER.warning(f"Cannot find photo of your pet. Defaulting to empty string.")
            self._photoLink = ""

        self._device = FiDevice(petJSON['device']['id'])
        self._device.setDeviceDetailsJSON(petJSON['device'])

    def __str__(self):
        return f"Last Updated - {self.lastUpdated} - Pet ID: {self.petId} Name: {self.name} Is Lost: {self.isLost} From: {self.homeCityState} ActivityType: {self.activityType} Located: {self.currLatitude},{self.currLongitude} Last Updated: {self.currStartTime}\n \
            using Device/Collar: {self._device}"
    
    # set the Pet's current location details
    def setCurrentLocation(self, activityJSON):
        activityType = activityJSON['__typename']
        self._activityType = activityType
        self._areaName = activityJSON['areaName']
        try:
            if activityType == PET_ACTIVITY_ONGOINGWALK:
                positionSize = len(activityJSON['positions'])
                self._currLongitude = float(activityJSON['positions'][positionSize-1]['position']['longitude'])
                self._currLatitude = float(activityJSON['positions'][positionSize-1]['position']['latitude'])
                self._currStartTime = datetime.datetime.fromisoformat(activityJSON['start'].replace('Z', '+00:00'))            
            else:          
                self._currLongitude = float(activityJSON['position']['longitude'])
                self._currLatitude = float(activityJSON['position']['latitude'])
                self._currStartTime = datetime.datetime.fromisoformat(activityJSON['start'].replace('Z', '+00:00'))
            try:
                self._currPlaceName = activityJSON['place']['name']
                self._currPlaceAddress = activityJSON['place']['address']
            except:
                LOGGER.warning("Could not set place, defaulting to Unknown")
                self._currPlaceName = "UNKNOWN"
                self._currPlaceAddress = "UNKNOWN"
            self._lastUpdated = datetime.datetime.now()
        except TryFiError as e:
            LOGGER.error(f"Unable to set values Current Location for Pet {self.name}.\nException: {e}\nwhile parsing {activityJSON}")
            raise TryFiError("Unable to set Pet Location Details")

    # set the Pet's current steps, goals and distance details for daily, weekly and monthly
    def setStats(self, activityJSONDaily, activityJSONWeekly, activityJSONMonthly):
        try:
            #distance is in metres
            self._dailyGoal = int(activityJSONDaily['stepGoal'])
            self._dailySteps = int(activityJSONDaily['totalSteps'])
            self._dailyTotalDistance = float(activityJSONDaily['totalDistance'])
        except TryFiError as e:
            LOGGER.error(f"Unable to set values Daily Stats for Pet {self.name}.\nException: {e}\nwhile parsing {activityJSONDaily}")
            raise TryFiError("Unable to set Pet Daily Stats")
        try:
            self._weeklyGoal = int(activityJSONWeekly['stepGoal'])
            self._weeklySteps = int(activityJSONWeekly['totalSteps'])
            self._weeklyTotalDistance = float(activityJSONWeekly['totalDistance'])
        except TryFiError as e:
            LOGGER.error(f"Unable to set values Weekly Stats for Pet {self.name}.\nException: {e}\nwhile parsing {activityJSONWeekly}")
            raise TryFiError("Unable to set Pet Weekly Stats")
        try:
            self._monthlyGoal = int(activityJSONMonthly['stepGoal'])
            self._monthlySteps = int(activityJSONMonthly['totalSteps'])
            self._monthlyTotalDistance = float(activityJSONMonthly['totalDistance'])
        except TryFiError as e:
            LOGGER.error(f"Unable to set values Monthly Stats for Pet {self.name}.\nException: {e}\nwhile parsing {activityJSONMonthly}")
            raise TryFiError("Unable to set Pet Monthly Stats")

        self._lastUpdated = datetime.datetime.now()


    # Update the Stats of the pet
    def updateStats(self, sessionId):
        try:
            pStatsJSON = query.getCurrentPetStats(sessionId,self.petId)
            self.setStats(pStatsJSON['dailyStat'],pStatsJSON['weeklyStat'],pStatsJSON['monthlyStat'])
            return True
        except Exception as e:
            LOGGER.error(f"Could not update stats for Pet {self.name}.\n{e}")

    # Update the Pet's GPS location
    def updatePetLocation(self, sessionId):
        try:
            pLocJSON = query.getCurrentPetLocation(sessionId,self.petId)
            self.setCurrentLocation(pLocJSON)
            return True
        except Exception as e:
            LOGGER.error(f"Could not update Pet: {self.name}'s location.\n{e}")
            return False
    
    # Update the device/collar details for this pet
    def updateDeviceDetails(self, sessionId):
        try:
            deviceJSON = query.getDevicedetails(sessionId, self.petId)
            self.device.setDeviceDetailsJSON(deviceJSON['device'])
            return True
        except Exception as e:
            LOGGER.error(f"Could not update Device/Collar information for Pet: {self.name}\n{e}")
            return False

    # Update all details regarding this pet
    def updateAllDetails(self, sessionId):
        self.updateDeviceDetails(sessionId)
        self.updatePetLocation(sessionId)
        self.updateStats(sessionId)

    # set the color code of the led light on the pet collar
    def setLedColorCode(self, sessionId, colorCode):
        try:
            moduleId = self.device.moduleId
            ledColorCode = int(colorCode)
            setColorJSON = query.setLedColor(sessionId, moduleId, ledColorCode)
            try:
                self.device.setDeviceDetailsJSON(setColorJSON['setDeviceLed'])
            except Exception as e:
                LOGGER.warning(f"Updated LED Color but could not get current status for Pet: {self.name}\nException: {e}")
            return True
        except Exception as e:
            LOGGER.error(f"Could not complete Led Color request:\n{e}")
            return False
    
    # turn on or off the led light. action = True will enable the light, false turns off the light
    def turnOnOffLed(self, sessionId, action):
        try:
            moduleId = self.device.moduleId
            onOffResponse = query.turnOnOffLed(sessionId, moduleId, action)
            try:
                self.device.setDeviceDetailsJSON(onOffResponse['updateDeviceOperationParams'])
            except Exception as e:
                LOGGER.warning(f"Action: {action} was successful however unable to get current status for Pet: {self.name}")
            return True
        except Exception as e:
            LOGGER.error(f"Could not complete LED request:\n{e}")
            return False

    # set the lost dog mode to Normal or Lost Dog. Action is true for lost dog and false for normal (not lost)
    def setLostDogMode(self, sessionId, action):
        try:
            moduleId = self.device.moduleId
            petModeResponse = query.setLostDogMode(sessionId, moduleId, action)
            try:
                self.device.setDeviceDetailsJSON(petModeResponse['updateDeviceOperationParams'])
            except Exception as e:
                LOGGER.warning(f"Action: {action} was successful however unable to get current status for Pet: {self.name}")
            return True
        except Exception as e:
            LOGGER.error(f"Could not complete Lost Dog Mode request:\n{e}")
            LOGGER.error(f"Could not complete turn on/off light where ledEnable is {action}.\nException: {e}")
            return False

    @property
    def device(self):
        return self._device
    @property
    def petId(self):
        return self._petId
    @property
    def name(self):
        return self._name
    @property
    def homeCityState(self):
        return self._homeCityState
    @property
    def yearOfBirth(self):
        return self._yearOfBirth
    @property
    def monthOfBirth(self):
        return self._monthOfBirth
    @property
    def dayOfBirth(self):
        return self._dayOfBirth
    @property
    def gender(self):
        return self._gender
    @property
    def weight(self):
        return self._weight
    @property
    def breed(self):
        return self._breed
    @property
    def photoLink(self):
        return self._photoLink
    @property
    def currLongitude(self):
        return self._currLongitude
    @property
    def currLatitude(self):
        return self._currLatitude
    @property
    def currStartTime(self):
        return self._currStartTime
    @property
    def currPlaceName(self):
        return self._currPlaceName
    @property
    def currPlaceAddress(self):
        return self._currPlaceAddress
    @property
    def currPlaceAddress(self):
        return self._currPlaceAddress
    @property
    def dailyGoal(self):
        return self._dailyGoal
    @property
    def dailySteps(self):
        return self._dailySteps
    @property
    def dailyTotalDistance(self):
        return self._dailyTotalDistance
    @property
    def weeklyGoal(self):
        return self._weeklyGoal
    @property
    def weeklySteps(self):
        return self._weeklySteps
    @property
    def weeklyTotalDistance(self):
        return self._weeklyTotalDistance
    @property
    def monthlyGoal(self):
        return self._monthlyGoal
    @property
    def monthlySteps(self):
        return self._monthlySteps
    @property
    def monthlyTotalDistance(self):
        return self._monthlyTotalDistance
    @property
    def lastUpdated(self):
        return self._lastUpdated
    @property
    def isLost(self):
        return self.device.isLost
    @property
    def activityType(self):
        return self._activityType
    @property
    def areaName(self):
        return self._areaName
    
    def getBirthDate(self):
        return datetime.datetime(self.yearOfBirth, self.monthOfBirth, self.dayOfBirth)
    
    def getDailySteps(self):
        return self.dailySteps
    
    def getDailyGoal(self):
        return self.dailyGoal

    def getDailyDistance(self):
        return self.dailyTotalDistance

    def getWeeklySteps(self):
        return self.weeklySteps
    
    def getWeeklyGoal(self):
        return self.weeklyGoal

    def getWeeklyDistance(self):
        return self.weeklyTotalDistance

    def getMonthlySteps(self):
        return self.monthlySteps
    
    def getMonthlyGoal(self):
        return self.monthlyGoal

    def getMonthlyDistance(self):
        return self.monthlyTotalDistance
        



from pytryfi.common import query
import datetime

class FiUser(object):
    def __init__(self, userId):
        self._userId = userId
        
    def setUserDetails(self, sessionId):
        response = query.getUserDetail(sessionId)
        self._email = response['email']
        self._firstName = response['firstName']
        self._lastName = response['lastName']
        self._phoneNumber = response['phoneNumber']
        self._lastUpdated = datetime.datetime.now()

    def __str__(self):
        return f"User ID: {self.userId} Name: {self.fullName} Email: {self.email}"
        
    @property
    def userId(self):
        return self._userId
    @property
    def email(self):
        return self._email
    @property
    def firstName(self):
        return self._firstName
    @property
    def lastName(self):
        return self._lastName
    @property
    def phoneNumber(self):
        return self._phoneNumber
    @property
    def fullName(self):
        return self.firstName + " "  + self.lastName
    @property
    def lastUpdated(self):
        return self._lastUpdated
    



from pytryfi.const import *
from pytryfi.exceptions import *
import requests
import logging

LOGGER = logging.getLogger(__name__)

def getUserDetail(sessionId):
    qString = QUERY_CURRENT_USER + FRAGMENT_USER_DETAILS
    response = query(sessionId, qString)
    LOGGER.debug(f"getUserDetails: {response}")
    return response['data']['currentUser']

def getPetList(sessionId):
    qString = QUERY_CURRENT_USER_FULL_DETAIL + FRAGMENT_USER_DETAILS \
        + FRAGMENT_USER_FULL_DETAILS + FRAGMENT_PET_PROFILE + FRAGEMENT_BASE_PET_PROFILE \
        + FRAGMENT_BASE_DETAILS + FRAGMENT_POSITION_COORDINATES + FRAGMENT_BREED_DETAILS \
        + FRAGMENT_PHOTO_DETAILS + FRAGMENT_DEVICE_DETAILS + FRAGMENT_LED_DETAILS + FRAGMENT_OPERATIONAL_DETAILS \
        + FRAGMENT_CONNECTION_STATE_DETAILS
    response = query(sessionId, qString)
    LOGGER.debug(f"getPetList: {response}")
    return response['data']['currentUser']['userHouseholds'][0]['household']['pets']

def getBaseList(sessionId):
    qString = QUERY_CURRENT_USER_FULL_DETAIL + FRAGMENT_USER_DETAILS \
        + FRAGMENT_USER_FULL_DETAILS + FRAGMENT_PET_PROFILE + FRAGEMENT_BASE_PET_PROFILE \
        + FRAGMENT_BASE_DETAILS + FRAGMENT_POSITION_COORDINATES + FRAGMENT_BREED_DETAILS \
        + FRAGMENT_PHOTO_DETAILS + FRAGMENT_DEVICE_DETAILS + FRAGMENT_LED_DETAILS + FRAGMENT_OPERATIONAL_DETAILS \
        + FRAGMENT_CONNECTION_STATE_DETAILS
    response = query(sessionId, qString)
    LOGGER.debug(f"getBaseList: {response}")
    return response['data']['currentUser']['userHouseholds'][0]['household']['bases']

def getCurrentPetLocation(sessionId, petId):
    qString = QUERY_PET_CURRENT_LOCATION.replace(VAR_PET_ID, petId) + FRAGMENT_ONGOING_ACTIVITY_DETAILS \
        + FRAGMENT_UNCERTAINTY_DETAILS + FRAGMENT_CIRCLE_DETAILS + FRAGMENT_LOCATION_POINT \
        + FRAGMENT_PLACE_DETAILS + FRAGMENT_USER_DETAILS + FRAGMENT_POSITION_COORDINATES
    response = query(sessionId, qString)
    LOGGER.debug(f"getCurrentPetLocation: {response}")
    return response['data']['pet']['ongoingActivity']

def getCurrentPetStats(sessionId, petId):
    qString = QUERY_PET_ACTIVITY.replace(VAR_PET_ID, petId) + FRAGMENT_ACTIVITY_SUMMARY_DETAILS
    response = query(sessionId, qString)
    LOGGER.debug(f"getCurrentPetStats: {response}")
    return response['data']['pet']

def getDevicedetails(sessionId, petId):
    qString = QUERY_PET_DEVICE_DETAILS.replace(VAR_PET_ID, petId) + FRAGMENT_PET_PROFILE + FRAGEMENT_BASE_PET_PROFILE + FRAGMENT_DEVICE_DETAILS + FRAGMENT_LED_DETAILS + FRAGMENT_OPERATIONAL_DETAILS + FRAGMENT_CONNECTION_STATE_DETAILS + FRAGMENT_USER_DETAILS + FRAGMENT_BREED_DETAILS + FRAGMENT_PHOTO_DETAILS
    response = query(sessionId, qString)
    LOGGER.debug(f"getDevicedetails: {response}")
    return response['data']['pet']

def setLedColor(sessionId, deviceId, ledColorCode):
    qString = MUTATION_SET_LED_COLOR + FRAGMENT_DEVICE_DETAILS + FRAGMENT_OPERATIONAL_DETAILS + FRAGMENT_CONNECTION_STATE_DETAILS + FRAGMENT_USER_DETAILS + FRAGMENT_LED_DETAILS
    qVariables = '{"moduleId":"'+deviceId+'","ledColorCode":'+str(ledColorCode)+'}'
    response = mutation(sessionId, qString, qVariables)
    LOGGER.debug(f"setLedColor: {response}")
    return response['data']

def turnOnOffLed(sessionId, moduleId, ledEnabled):
    qString = MUTATION_DEVICE_OPS + FRAGMENT_DEVICE_DETAILS + FRAGMENT_OPERATIONAL_DETAILS + FRAGMENT_CONNECTION_STATE_DETAILS + FRAGMENT_USER_DETAILS + FRAGMENT_LED_DETAILS
    qVariables = '{"input": {"moduleId":"'+moduleId+'","ledEnabled":'+str(ledEnabled).lower()+'}}'
    response = mutation(sessionId, qString, qVariables)
    LOGGER.debug(f"turnOnOffLed: {response}")
    return response['data']

def setLostDogMode(sessionId, moduleId, action):
    if action:
        mode = PET_MODE_LOST
    else:
        mode = PET_MODE_NORMAL
    qString = MUTATION_DEVICE_OPS + FRAGMENT_DEVICE_DETAILS + FRAGMENT_OPERATIONAL_DETAILS + FRAGMENT_CONNECTION_STATE_DETAILS + FRAGMENT_USER_DETAILS + FRAGMENT_LED_DETAILS
    qVariables = '{"input": {"moduleId":"'+moduleId+'","mode":"'+mode+'"}}'
    response = mutation(sessionId, qString, qVariables)
    LOGGER.debug(f"setLostDogMode: {response}")
    return response['data']

def getGraphqlURL():
    return API_HOST_URL_BASE + API_GRAPHQL

def mutation(sessionId, qString, qVariables):
    jsonObject = None
    url = getGraphqlURL()
    params = {"query": qString, "variables": qVariables}
    jsonObject = execute(url, sessionId, params=params, method='POST').json()
    return jsonObject

def query(sessionId, qString):
    jsonObject = None
    url = getGraphqlURL()
    params={'query': qString}
    jsonObject = execute(url, sessionId, params=params).json()
    return jsonObject

def execute(url, sessionId, method='GET', params=None, cookies=None):
    response = None
    try:
        if method == 'GET':
            response = sessionId.get(url, params=params)
        elif method == 'POST':
            response = sessionId.post(url, data=params)
        else:
            raise TryFiError(f"Method Passed was invalid: {method}")
    except requests.RequestException as e:
        raise requests.RequestException(e)
    return response
    


import logging
import requests

from pytryfi.const import (API_HOST_URL_BASE, API_LOGIN, API_GRAPHQL, PYTRYFI_VERSION)
from pytryfi.fiUser import FiUser
from pytryfi.fiPet import FiPet
from pytryfi.fiBase import FiBase
from pytryfi.common import query
from pytryfi.const import SENTRY_URL
import sentry_sdk



LOGGER = logging.getLogger(__name__)

class PyTryFi(object):
    """base object for TryFi"""

    def __init__(self, username=None, password=None):
        sentry = sentry_sdk.init(
                SENTRY_URL,
                release=PYTRYFI_VERSION,
                traces_sample_rate=1.0,
            )
        self._api_host = API_HOST_URL_BASE
        self._session = requests.Session()
        self._user_agent = "pyTryFi"
        self._username = username
        self._password = password
        self.login()

        self._currentUser = FiUser(self._userId)
        self._currentUser.setUserDetails(self._session)
        petListJSON = query.getPetList(self._session)
        self._pets = []
        for pet in petListJSON:
            p = FiPet(pet['id'])
            p.setPetDetailsJSON(pet)
            #get the current location and set it
            pLocJSON = query.getCurrentPetLocation(self._session,p._petId)
            p.setCurrentLocation(pLocJSON)
            #get the daily, weekly and monthly stats and set
            pStatsJSON = query.getCurrentPetStats(self._session,p._petId)
            p.setStats(pStatsJSON['dailyStat'],pStatsJSON['weeklyStat'],pStatsJSON['monthlyStat'])
            LOGGER.debug(f"Adding Pet: {p._name} with Device: {p._device._deviceId}")
            self._pets.append(p)
        
        self._bases = []
        baseListJSON = query.getBaseList(self._session)
        for base in baseListJSON:
            b = FiBase(base['baseId'])
            b.setBaseDetailsJSON(base)
            LOGGER.debug(f"Adding Base: {b._name} Online: {b._online}")
            self._bases.append(b)

    def __str__(self):
        instString = f"Username: {self.username}"
        userString = f"{self.currentUser}"
        baseString = ""
        petString = ""
        for b in self.bases:
            baseString = baseString + f"{b}"
        for p in self.pets:
            petString = petString + f"{p}"
        return f"TryFi Instance - {instString}\n Pets in Home:\n {petString}\n Bases In Home:\n {baseString}"
        
    #refresh pet details for all pets
    def updatePets(self):
        petListJSON = query.getPetList(self._session)
        updatedPets = []
        for pet in petListJSON:
            p = FiPet(pet['id'])
            p.setPetDetailsJSON(pet)
            #get the current location and set it
            pLocJSON = query.getCurrentPetLocation(self._session,p._petId)
            p.setCurrentLocation(pLocJSON)
            #get the daily, weekly and monthly stats and set
            pStatsJSON = query.getCurrentPetStats(self._session,p._petId)
            p.setStats(pStatsJSON['dailyStat'],pStatsJSON['weeklyStat'],pStatsJSON['monthlyStat'])
            LOGGER.debug(f"Adding Pet: {p._name} with Device: {p._device._deviceId}")
            updatedPets.append(p)
        self._pets = updatedPets

    def updatePetObject(self, petObj):
        petId = petObj.petId
        count = 0
        for p in self.pets:
            if p.petId == petId:
                self._pets.pop(count)
                self._pets.append(petObj)
                LOGGER.debug(f"Updating Existing Pet: {petId}")
                break
            count = count + 1

    # return the pet object based on petId
    def getPet(self, petId):
        for p in self.pets:
            if petId == p.petId:
                return p
        LOGGER.error(f"Cannot find Pet: {petId}")
        return None
    
    #refresh base details
    def updateBases(self):
        updatedBases = []
        baseListJSON = query.getBaseList(self._session)
        for base in baseListJSON:
            b = FiBase(base['baseId'])
            b.setBaseDetailsJSON(base)
            updatedBases.append(b)
        self._bases = updatedBases

    # return the pet object based on petId
    def getBase(self, baseId):
        for b in self.bases:
            if baseId == b.baseId:
                return b
        LOGGER.error(f"Cannot find Base: {baseId}")
        return None

    def update(self):
        self.updateBases()
        self.updatePets()

    @property
    def currentUser(self):
        return self._currentUser
    @property
    def pets(self):
        return self._pets
    @property
    def bases(self):
        return self._bases
    @property
    def username(self):
        return self._username
    @property
    def session(self):
        return self._session
    @property
    def cookies(self):
        return self._cookies
    @property
    def userID(self):
        return self._userID
    @property
    def session(self):
        return self._session

    # login to the api and get a session
    def login(self):
        url = API_HOST_URL_BASE + API_LOGIN
        params={
                'email' : self._username,
                'password' : self._password,
            }
        
        LOGGER.debug(f"Logging into TryFi")
        try:
            response = self._session.post(url, data=params)
            response.raise_for_status()
        except requests.RequestException as e:
            LOGGER.error(f"Cannot login, error: ({e})")
            raise requests.RequestException(e)
        error = None
        try:
            error = response.json()['error']
        except:
            error = None
        if error or not response.ok:
            errorMsg = error['message']
            LOGGER.error(f"Cannot login, response: ({response.status_code}): {errorMsg} ")
            raise Exception("TryFiLoginError")
        #storing cookies but don't need them. Handled by session mgmt
        self._cookies = response.cookies
        #store unique userId from login for future use
        self._userId = response.json()['userId']
        self._sessionId = response.json()['sessionId']
        LOGGER.debug(f"Successfully logged in. UserId: {self._userId}")


import logging
import datetime
from pytryfi.ledColors import ledColors
from pytryfi.const import PET_MODE_NORMAL, PET_MODE_LOST

LOGGER = logging.getLogger(__name__)

class FiDevice(object):
    def __init__(self, deviceId):
        self._deviceId = deviceId
    
    def setDeviceDetailsJSON(self, deviceJSON):
        self._moduleId = deviceJSON['moduleId']
        self._buildId = deviceJSON['info']['buildId']
        self._batteryPercent = int(deviceJSON['info']['batteryPercent'])
        self._isCharging = bool(deviceJSON['info']['isCharging'])
        self._batteryHealth = deviceJSON['info']['batteryHealth']
        self._ledOffAt = self.setLedOffAtDate(deviceJSON['operationParams']['ledOffAt'])
        self._ledOn = self.getAccurateLEDStatus( bool(deviceJSON['operationParams']['ledEnabled']))
        self._mode = deviceJSON['operationParams']['mode']
        self._ledColor = deviceJSON['ledColor']['name']
        self._ledColorHex = deviceJSON['ledColor']['hexCode']
        self._connectionStateDate = datetime.datetime.fromisoformat(str(deviceJSON['lastConnectionState']['date']).replace('Z', '+00:00'))
        self._connectionStateType = deviceJSON['lastConnectionState']['__typename']
        self._availableLedColors = []
        self._lastUpdated = datetime.datetime.now()
        for cString in deviceJSON['availableLedColors']:
            c = ledColors(int(cString['ledColorCode']),cString['hexCode'], cString['name'] )
            self._availableLedColors.append(c)

    def __str__(self):
        return f"Last Updated - {self.lastUpdated} - Device ID: {self.deviceId} Device Mode: {self.mode} Battery Left: {self.batteryPercent}% LED State: {self.ledOn} Last Connected: {self.connectionStateDate} by: {self.connectionStateType}"

    @property
    def deviceId(self):
        return self._deviceId
    @property
    def moduleId(self):
        return self._moduleId
    @property
    def mode(self):
        return self._mode
    @property
    def buildId(self):
        return self._buildId
    @property
    def batteryPercent(self):
        return self._batteryPercent
    @property
    def batteryHealth(self):
        return self._batteryHealth
    @property
    def isCharging(self):
        return self._isCharging
    @property
    def ledOn(self):
        return self._ledOn
    @property
    def ledOffAt(self):
        return self._ledOffAt
    @property
    def ledColor(self):
        return self._ledColor
    @property
    def ledColorHex(self):
        return self._ledColorHex
    @property
    def connectionStateDate(self):
        return self._connectionStateDate
    @property
    def connectionStateType(self):
        return self._connectionStateType
    @property
    def availableLedColors(self):
        return self._availableLedColors
    @property
    def lastUpdated(self):
        return self._lastUpdated
    @property
    def isLost(self):
        if self._mode == PET_MODE_LOST:
            return True
        else:
            return False

#This is created because if TryFi automatically turns off the LED, the status is still set to true in the api.
#This will compare the dates to see if the current date/time is greater than the turnoffat time in the api.
    def getAccurateLEDStatus(self, ledStatus):
        if ledStatus is False:
            LOGGER.debug("getAccurateLedStatus: LED Status is False")
            return False
        else:
            LOGGER.debug("getAccurateLedStatus: LED Status is True... comparing date/times")
            currentDateTime = datetime.datetime.now(datetime.timezone.utc)
            if currentDateTime > self.ledOffAt:
                LOGGER.debug(f"Current datetime: {currentDateTime} is greater than ledOffAt: {self.ledOffAt}, Returning False")
                return False
            else:
                LOGGER.debug(f"Current datetime: {currentDateTime} is less than ledOffAt: {self.ledOffAt}, Returning False")
                return True
#Created function to return a date/time regardless.
    def setLedOffAtDate(self, ledOffAt):
        if ledOffAt == None:
            ## if object is null, return current date/time in UTC
            LOGGER.debug("LedOffAt is None, returning current datetime in UTC")
            return datetime.datetime.now(datetime.timezone.utc)
        else:
            LOGGER.debug(f"LedOffAt has date/time of {ledOffAt}. Returning this in ISO Format.")
            return datetime.datetime.fromisoformat(str(ledOffAt).replace('Z', '+00:00'))


SENTRY_URL = "https://c7618923f758480ca2af05a21123f855@o580516.ingest.sentry.io/5735605"

PYTRYFI_VERSION = "0.0.11"

API_HOST_URL_BASE   = "https://api.tryfi.com"
API_GRAPHQL         = "/graphql"
API_LOGIN           = "/auth/login"

PET_MODE_NORMAL = "NORMAL"
PET_MODE_LOST = "LOST_DOG"
PET_ACTIVITY_ONGOINGWALK = "OngoingWalk"
PET_ACTIVITY_ONGOINGREST = "OngoingRest"
PET_ACTIVITY_WALK = "Walk"
PET_ACTIVITY_REST = "Rest"

VAR_PET_ID = "__PET_ID__"

QUERY_CURRENT_USER  = "query {\n  currentUser {\n    ...UserDetails\n  }\n}\n"
QUERY_CURRENT_USER_FULL_DETAIL  = "query {\n  currentUser {\n    ...UserFullDetails\n  }\n}\n"
QUERY_PET_CURRENT_LOCATION = "query {\n\n  pet (id: \""+VAR_PET_ID+"\") {\n    ongoingActivity(walksVersion: 1) {\n      __typename\n      ...OngoingActivityDetails\n    }\n  }\n}\n"
QUERY_PET_ACTIVITY = "query {\n\n  pet (id: \""+VAR_PET_ID+"\") {\n   \n    dailyStat: currentActivitySummary (period: DAILY) {\n      ...ActivitySummaryDetails\n    }\n    weeklyStat: currentActivitySummary (period: WEEKLY) {\n      ...ActivitySummaryDetails\n    }\n    monthlyStat: currentActivitySummary (period: MONTHLY) {\n      ...ActivitySummaryDetails\n    }\n  }\n}\n"
QUERY_PET_DEVICE_DETAILS = "query {\n  pet (id: \""+VAR_PET_ID+"\") {\n    __typename\n    ...PetProfile\n  }\n}\n"

FRAGMENT_USER_DETAILS = "fragment UserDetails on User {\n  __typename\n   id\n  email\n  firstName\n  lastName\n  phoneNumber\n  fiNewsNotificationsEnabled\n  chipReseller {\n    __typename\n    id\n  }\n}\n"
FRAGMENT_USER_FULL_DETAILS = "fragment UserFullDetails on User {\n  __typename\n  ...UserDetails\n  userHouseholds {\n    __typename\n    household {\n      __typename\n      pets {\n        __typename\n        ...PetProfile\n      }\n      bases {\n        __typename\n        ...BaseDetails\n      }\n    }\n  }\n}\n"
FRAGEMENT_BASE_PET_PROFILE = "fragment BasePetProfile on BasePet {\n  __typename\n  id\n  name\n  homeCityState\n  yearOfBirth\n  monthOfBirth\n  dayOfBirth\n  gender\n  weight\n  isPurebred\n  breed {\n    __typename\n    ...BreedDetails\n  }\n  photos {\n    __typename\n    first {\n      __typename\n      ...PhotoDetails\n    }\n    items {\n      __typename\n      ...PhotoDetails\n    }\n  }\n  instagramHandle\n  \n}\n"
FRAGMENT_BREED_DETAILS = "fragment BreedDetails on Breed {\n  __typename\n  id\n  name\n  popularityScore\n}\n"
FRAGMENT_PHOTO_DETAILS = "fragment PhotoDetails on Photo {\n  __typename\n  id\n  caption\n  date\n  likeCount\n  liked\n  image {\n    __typename\n    fullSize\n  }\n}\n"
FRAGMENT_PET_PROFILE = "fragment PetProfile on Pet {\n  __typename\n  ...BasePetProfile\n  chip {\n    __typename\n    shortId\n  }\n  device {\n    __typename\n    ...DeviceDetails\n  }\n}\n"
FRAGMENT_DEVICE_DETAILS = "fragment DeviceDetails on Device {\n  __typename\n  id\n  moduleId\n  info\n  subscriptionId\n  hasActiveSubscription\n  hasSubscriptionOverride\n  nextLocationUpdateExpectedBy\n  operationParams {\n    __typename\n    ...OperationParamsDetails\n  }\n  lastConnectionState {\n    __typename\n    ...ConnectionStateDetails\n  }\n  ledColor {\n    __typename\n    ...LedColorDetails\n  }\n  availableLedColors {\n    __typename\n    ...LedColorDetails\n  }\n}\n"
FRAGMENT_LED_DETAILS = "fragment LedColorDetails on LedColor {\n  __typename\n  ledColorCode\n  hexCode\n  name\n}\n"
FRAGMENT_CONNECTION_STATE_DETAILS = "fragment ConnectionStateDetails on ConnectionState {\n  __typename\n  date\n  ... on ConnectedToUser {\n    user {\n      __typename\n      ...UserDetails\n    }\n  }\n  ... on ConnectedToBase {\n    chargingBase {\n      __typename\n      id\n    }\n  }\n  ... on ConnectedToCellular {\n    signalStrengthPercent\n  }\n  ... on UnknownConnectivity {\n    unknownConnectivity\n  }\n}\n"
FRAGMENT_OPERATIONAL_DETAILS = "fragment OperationParamsDetails on OperationParams {\n  __typename\n  mode\n  ledEnabled\n  ledOffAt\n}\n"
FRAGMENT_BASE_DETAILS = "fragment BaseDetails on ChargingBase {\n  __typename\n  baseId\n  name\n  position {\n    __typename\n    ...PositionCoordinates\n  }\n  infoLastUpdated\n  networkName\n  online\n  onlineQuality\n}\n"
FRAGMENT_POSITION_COORDINATES = "fragment PositionCoordinates on Position {\n  __typename\n  latitude\n  longitude\n}\n"
FRAGMENT_ONGOING_ACTIVITY_DETAILS = "fragment OngoingActivityDetails on OngoingActivity {\n  __typename\n  start\n  presentUser {\n    __typename\n    ...UserDetails\n  }\n  areaName\n  lastReportTimestamp\n  obfuscatedReason\n  totalSteps\n  uncertaintyInfo {\n    __typename\n    ...UncertaintyInfoDetails\n  }\n  ... on OngoingWalk {\n    distance\n    positions {\n      __typename\n      ...LocationPoint\n    }\n    path {\n      __typename\n      ...PositionCoordinates\n    }\n  }\n  ... on OngoingRest {\n    position {\n      __typename\n      ...PositionCoordinates\n    }\n    place {\n      __typename\n      ...PlaceDetails\n    }\n  }\n}\n"
FRAGMENT_UNCERTAINTY_DETAILS = "fragment UncertaintyInfoDetails on UncertaintyInfo {\n  __typename\n  areaName\n  updatedAt\n  circle {\n    __typename\n    ...CircleDetails\n  }\n}\n"
FRAGMENT_CIRCLE_DETAILS = "fragment CircleDetails on Circle {\n  __typename\n  radius\n  latitude\n  longitude\n}\n"
FRAGMENT_LOCATION_POINT = "fragment LocationPoint on Location {\n  __typename\n  date\n  errorRadius\n  position {\n    __typename\n    ...PositionCoordinates\n  }\n}\n"
FRAGMENT_PLACE_DETAILS = "fragment PlaceDetails on Place {\n  __typename\n  id\n  name\n  address\n  position {\n    __typename\n    ...PositionCoordinates\n  }\n  radius\n}\n"
FRAGMENT_ACTIVITY_SUMMARY_DETAILS = "fragment ActivitySummaryDetails on ActivitySummary {\n  __typename\n  start\n  end\n  totalSteps\n  stepGoal\n  dailySteps {\n    __typename\n    date\n    totalSteps\n    stepGoal\n  }\n  totalDistance\n}\n"

MUTATION_DEVICE_OPS = "mutation UpdateDeviceOperationParams($input: UpdateDeviceOperationParamsInput!) {\n  updateDeviceOperationParams(input: $input) {\n    __typename\n    ...DeviceDetails\n  }\n}\n"
MUTATION_SET_LED_COLOR = "mutation SetDeviceLed($moduleId: String!, $ledColorCode: Int!) {\n  setDeviceLed(moduleId: $moduleId, ledColorCode: $ledColorCode) {\n    __typename\n    ...DeviceDetails\n  }\n}\n"


#!/usr/bin/env python

import os
import sys
from setuptools import setup, find_packages
from pytryfi.const import PYTRYFI_VERSION

if sys.argv[-1] == 'compile':
    os.system('python setup.py bdist_wheel')
    sys.exit()

# read the contents of your README file
from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

pkg = find_packages("pytryfi")
print(pkg)

setup(
    name="pytryfi", # Replace with your own username
    version=PYTRYFI_VERSION,
    author="Steve Babcock",
    author_email="steve.w.babcock@gmail.com",
    description="Python Interface for TryFi Dog Collars",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/sbabcock23/pytryfi",
    packages={'pytryfi', 'pytryfi/common'},
    include_package_data=True,
    classifiers=[ "Programming Language :: Python :: 3",
                    "License :: OSI Approved :: Apache Software License",
                    "Operating System :: OS Independent",
    ],
    install_requires=[
        'requests',
        'sentry-sdk',
    ],
    python_requires='>=3.6',
)


class ledColors(object):
    def __init__(self, ledColorCode, hexCode, name):
        self._ledColorCode = ledColorCode
        self._hexCode = hexCode
        self._name = name
    
    def __str__(self):
        return f"Color: {self.name} Hex Code: {self.hexCode} Color Code: {self.ledColorCode}"
    
    @property
    def name(self):
        return self._name
    @property
    def ledColorCode(self):
        return self._ledColorCode
    @property
    def hexCode(self):
        return self._hexCode


import datetime

class FiBase(object):
    def __init__(self, baseId):
        self._baseId = baseId
    
    def setBaseDetailsJSON(self, baseJSON):
        self._name = baseJSON['name']
        self._latitude = baseJSON['position']['latitude']
        self._longitude = baseJSON['position']['longitude']
        self._online = baseJSON['online']
        self._onlineQuality = baseJSON['onlineQuality']
        self._lastUpdated = baseJSON['infoLastUpdated']
        self._networkName = baseJSON['networkName']
        self._lastUpdated = datetime.datetime.now()

    def __str__(self):
        return f"Last Updated - {self.lastUpdated} - Base ID: {self.baseId} Name: {self.name} Online Status: {self.online} Wifi Network: {self.networkname} Located: {self.latitude},{self.longitude}"
        
    @property
    def baseId(self):
        return self._baseId
    @property
    def name(self):
        return self._name
    @property
    def latitude(self):
        return self._latitude
    @property
    def longitude(self):
        return self._longitude
    @property
    def online(self):
        return self._online
    @property
    def onlineQuality(self):
        return self._onlineQuality
    @property
    def lastupdate(self):
        return self._lastUpdated
    @property
    def networkname(self):
        return self._networkName
    @property
    def lastUpdated(self):
        return self._lastUpdated